#ifndef _LIBLINEAR_H
#define _LIBLINEAR_H
#define _DENSE_REP 1

#ifdef __cplusplus
extern "C" {
#endif
    
    
struct problem
{
  int l,n;
  int *y;
  double **x;
  double bias;
};



enum { L2R_LR, L2R_L2LOSS_SVC_DUAL, L2R_L2LOSS_SVC, L2R_L1LOSS_SVC_DUAL, MCSVM_CS, L1R_L2LOSS_SVC, L1R_LR }; /* solver_type */

struct parameter
{
	int solver_type;

	/* these are for training only */
	double eps;	        /* stopping criteria */
	double C;
	int nr_weight;
	int *weight_label;
	double* weight;
};

struct model
{
	struct parameter param;
	int nr_class;		/* number of classes */
	int nr_feature;
	double *w;
	int *label;		/* label of each class */
	double bias;
};

struct model* train(const struct problem *prob, const struct parameter *param);
void cross_validation(const struct problem *prob, const struct parameter *param, int nr_fold, int *target);

#ifdef _DENSE_REP
int predict_values(const struct model *model_, const double *x, double* dec_values);
int predict(const struct model *model_, const double *x);
int predict_probability(const struct model *model_, const double *x, double* prob_estimates);
#else
int predict_values(const struct model *model_, const struct feature_node *x, double* dec_values);
int predict(const struct model *model_, const struct feature_node *x);
int predict_probability(const struct model *model_, const struct feature_node *x, double* prob_estimates);
#endif

int save_model(const char *model_file_name, const struct model *model_);
struct model *load_model(const char *model_file_name);

int get_nr_feature(const struct model *model_);
int get_nr_class(const struct model *model_);
void get_labels(const struct model *model_, int* label);

void destroy_model(struct model *model_);
void destroy_param(struct parameter *param);
const char *check_parameter(const struct problem *prob, const struct parameter *param);
extern void (*liblinear_print_string) (const char *);

#ifdef __cplusplus
}
#endif

#endif /* _LIBLINEAR_H */

